/* eslint-disable @typescript-eslint/no-explicit-any */
import { Canvas, Rect, rect } from "@shopify/react-native-skia";
import React, { useState, useEffect } from "react";
import { View, StyleSheet } from "react-native";
import Animated, {
  useAnimatedStyle,
  useDerivedValue,
  useSharedValue,
} from "react-native-reanimated";

import { useLoop } from "../../components/Animations";

import { BreatheScreen } from "./BreatheScreen";

const Screen = () => {
  const progress = useLoop({ duration: 3000 });
  const style = useAnimatedStyle(() => ({
    width: progress.value * 100,
    height: 100,
  }));
  const size = useSharedValue({ width: 0, height: 0 });
  const rct = useDerivedValue(() =>
    rect(0, 0, size.value.width, size.value.height / 2)
  );

  // Adding more elements to increase complexity
  const moreElements = Array.from({ length: 5 }).map((_, index) => (
    <Rect
      key={index}
      rect={rct}
      color={`rgba(${getRandomInt(0, 255)}, ${getRandomInt(
        0,
        255
      )}, ${getRandomInt(0, 255)}, 0.5)`}
    />
  ));

  return (
    <Animated.View style={style}>
      <Canvas style={StyleSheet.absoluteFill} onSize={size}>
        <BreatheScreen />
        {moreElements}
      </Canvas>
    </Animated.View>
  );
};

const getRandomInt = (min: number, max: number) => {
  return Math.floor(Math.random() * (max - min + 1)) + min;
};

export const StressTest = () => {
  const [screens, setScreens] = useState<any>([]);

  const addScreens = (count: number) => {
    setScreens((prevScreens: any) => [
      ...prevScreens,
      ...Array.from({ length: count }).map(() => ({
        id: Date.now() + Math.random(),
        component: <Screen />,
      })),
    ]);
  };

  const removeScreens = (count: number) => {
    setScreens((prevScreens: any) => {
      if (prevScreens.length === 0) {
        return prevScreens;
      }
      const indicesToRemove = new Set<number>();
      while (
        indicesToRemove.size < count &&
        indicesToRemove.size < prevScreens.length
      ) {
        indicesToRemove.add(Math.floor(Math.random() * prevScreens.length));
      }
      return prevScreens.filter(
        (_: any, index: number) => !indicesToRemove.has(index)
      );
    });
  };

  useEffect(() => {
    const intervalId = setInterval(() => {
      const randomCount = getRandomInt(1, 5); // Adjust the range to control the number of screens added/removed at once
      if (Math.random() > 0.5) {
        addScreens(randomCount);
      } else {
        removeScreens(randomCount);
      }
    }, 1000); // Faster rate

    return () => clearInterval(intervalId);
  }, []);

  return (
    <View style={{ flex: 1 }}>
      {screens.map((screen: any) => (
        <React.Fragment key={screen.id}>{screen.component}</React.Fragment>
      ))}
    </View>
  );
};
